#!/usr/bin/python
import sys
import os
import eon_read
import eon_extract
from optparse import OptionParser
import substrate
import math
import numpy
import pickle
import collections 
import kmc
import msd
import config
import logging
logging.basicConfig(level=logging.INFO,
            filename=os.path.join(".", "kmc.log"),filemode="w",
            format="%(asctime)s %(levelname)s:%(name)s:%(message)s",
            datefmt="%F %T")
logging.raiseExceptions = False
logger = logging.getLogger('kmc_diffusion')      
        


#################################################################################################################################
def main():
    
    config.init()

    #set random state
    #if there is a randomstatefile in the current folder, use that one.
    try:
        with open(config.path_randomstate) as randomstatefile:
            logger.info("Setting randomstate from file")
            randomstate = pickle.load(randomstatefile)
            numpy.random.set_state(randomstate)
        randomstatefile.close()    
    except IOError as e:
        logger.info("Setting seed to %s" % (repr(config.main_random_seed) ) )
        numpy.random.seed(config.main_random_seed)
        #save randomstate to file
        randomstatefile=eon_read._open_file_writing(config.path_randomstate)    
        randomstate = numpy.random.get_state()
        pickle.dump(randomstate,randomstatefile)
        randomstatefile.close()


    #read statelist (no configurations)
    statelist = eon_read.read_statelist(path=config.path_states, read_reactant=True, read_processes=True, read_configurations=False, read_all=False, read_configs_unknown=False)
    numstates = len(statelist)    
    
    initialstatelist = []
    
    #do we need to evaluate substrates?
    if config.diffusion_evaluate_substrates:
        #read substrates from disk?
        if config.diffusion_read_substrates:
            substratelist = substrate.read_substrates(statelist, os.path.join( config.path_root,'substrates' ) )
        else:
            sys.stderr.write("Determining substrates...\n")
            substratelist = substrate.determine_substrates(statelist,config.diffusion_addmolid, config.diffusion_trans_cutoff, config.diffusion_angle_cutoff)
        for isub in range(len( substratelist) ):
            sys.stderr.write("Substrate %d:\n%s\n" % (isub,repr(substratelist[isub] ) ) )
        res = raw_input("Continue? (y/N)").lower()          
        if res[0] !='y':
            exit(1)
        res = raw_input("Which substrates should be used for simulation? (enter comma separated list)").lower()

        try:
            validsubstrates = [int(i) for i in res.split(',') ]
            for isub in validsubstrates:
                initialstatelist += substratelist[isub]
        except ValueError:
            sys.stderr.write("Could not parse substrate list %s\n" % (res) )
            exit(1)
        except IndexError:
            sys.stderr.write("Invalid substrateid entered\n")
            exit(1)
    elif config.kmc_read_initial_statelist:
        try:
            initialstatelistfile = eon_read._open_file_reading(os.path.join( config.path_root,'initial_states.dat') )
            initialstatelist = [ int(i.strip('[,]')) for i in initialstatelistfile.readline().split() ]
            logger.info("Initial statelist read from file: %s\n" % repr(initialstatelist) )
            for stateid in initialstatelist:
                if stateid >= len(statelist):
                    sys.stderr.write("Removing state invalid state %d from initialstatelist\n" % stateid)
                    initialstatelist.remove(stateid)
                while initialstatelist.count(stateid) > 1:
                    initialstatelist.remove(stateid)
            initialstatelistfile.close()
            if len(initialstatelist)==0:
                sys.stderr.write("No state in initialstatelistfile read from disk\n")
                raise ValueError
        except:
            sys.stderr.write("Defaulting to complete statelist. Okay?\n")
            res = raw_input("Continue? (y/N)").lower() 
            if res[0] !='y':
                exit(1)  
            initialstatelist = range(len(statelist))
    else:
        initialstatelist = range(len(statelist))      
        
    print "initial substratelist:", initialstatelist          

    #initialize simulation       
    simulation = kmc.kmcSim(statelist, initialstatelist)    
    
    if config.diffusion_evaluate_substrates:
        substrateid = statelist[simulation.stateid]['substrateid'] 


    #initialize diffusion tracking
    if config.diffusion_calc:
        diffusion = msd.msdObject()
        msdfile = open("msd.dat","w")


    sys.stderr.write("Starting KMC from state %d\n" % simulation.stateid)
    
    istep = 0
    while ( istep < config.max_kmc_steps ):
        if (istep % 5000) == 0:
            sys.stderr.write("KMC step %d, state = %d, time =%e\n" % (istep,simulation.stateid,simulation.totaltime) )
            
        if config.kmc_max_steps_onerun:
            if (istep % config.kmc_max_steps_onerun) ==0:
                sys.stderr.write("Restarting simulation, max steps onerun reached\n")
                simulation.randomStart()
                if config.diffusion_calc:
                    diffusion._newSegment(exceededMaxT = False)       
                            
        #if (istep % 100000) == 0 and config.sb_on:
        #    simulation.resetSuperbasins()
        
        #make step
        simulation.onestep()
        istep +=1
        
        #check that we don't move to a new state
        if simulation.stateid==-1:
            logger.info("ERROR: moved to new state: process %d from state %d" % (simulation.procid,simulation.reactant_id))
            if config.kmc_restart_new_state:
                simulation.randomStart()
                if config.diffusion_calc:
                    diffusion._newSegment(exceededMaxT = False)                    
            else:
                if config.diffusion_calc:
                    diffusion.write(msdfile)
                    msdfile.close() 
                exit(1)
            continue    
        
        #check that we don't enter a state without confidence:
        if config.kmc_calc_confidence:
            if not statelist[ simulation.stateid ]['confident']:
                logger.info("ERROR: moved to state %d with too low confidence level %f" % (simulation.stateid,statelist[simulation.stateid]['confidence']))
                simulation.randomStart()
                if config.diffusion_calc:
                    diffusion._newSegment(exceededMaxT = False)    
                continue                                         

        
        #check if we stay in the substrate
        if config.diffusion_evaluate_substrates:
            if statelist[simulation.stateid]['substrateid'] != substrateid:
                logger.info("Changing substrate!")
                simulation.randomStart()
                if config.diffusion_calc:
                    diffusion._newSegment(exceededMaxT = False)
                continue
        
        #register the msd
        if config.diffusion_calc:
            displacement = simulation.moleculedisplacement()
            diffusion.registerStep(displacement,simulation.timestep)
            
        #keep track of binding energy
        if config.kmc_calc_binding_energy:
            simulation.bindinge += simulation.timestep * statelist[ simulation.previousstate ]['bindinge']
            print "state = %d, be = %f, t = %f\n" % (simulation.previousstate,statelist[ simulation.previousstate ]['bindinge'],simulation.timestep)

                

    if config.kmc_calc_binding_energy and config.diffusion_calc:
        logger.info("Average binding energy of %f seconds was %f meV\n" % (diffusion.t,simulation.bindinge/diffusion.t) )

    if config.kmc_track_visited_states:
        simulation.write_num_visits()

    if config.diffusion_calc:
        diffusion.write(msdfile)
        msdfile.close() 

    logging.shutdown()        

if __name__ == "__main__":
    main()
